/*
 * Renderer.h
 *
 * Copyright (C) 2009 Matthias Schiffer <matthias@gamezock.de>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef ZOOM_RENDERER_H_
#define ZOOM_RENDERER_H_

#include "gl.h"
#include "BSPTree.h"
#include "Shader.h"


namespace Zoom {

class Renderer {
  public:
    Renderer();

    void render(const BSPTree &tree);

    template <typename T>
    void render(const T &triangles) {
      Shader::enable(ambientShader);
      glBlendFunc(GL_ONE, GL_ZERO);
      glDepthFunc(GL_LEQUAL);

      typename T::const_iterator t = triangles.begin();
      if(t == triangles.end())
        return;

      useTexture(t->triangle.getTexture());

      glBegin(GL_TRIANGLES);
      for(; t != triangles.end(); ++t) {
        renderTriangle(t->triangle);
      }
      glEnd();

      Shader::enable(lightShader);
      glBlendFunc(GL_ONE, GL_ONE);
      glDepthFunc(GL_EQUAL);

      t = triangles.begin();

      useTexture(t->triangle.getTexture());

      glBegin(GL_TRIANGLES);
      for(; t != triangles.end(); ++t) {
        renderTriangle(t->triangle);
      }
      glEnd();
    }

    struct TextureSorter {
      bool operator() (const BSPTree::TriangleRecord &t1, const BSPTree::TriangleRecord &t2) {
        return (t1.triangle.getTexture() < t2.triangle.getTexture());
      }
    };

  private:
    void renderTriangle(const Triangle &t);
    void useTexture(unsigned texture);

    class RenderVisitor {
      public:
        RenderVisitor(Renderer *renderer0) : renderer(renderer0) {}

        void operator() (const BSPTree::TriangleRecord &t) const {
          renderer->renderTriangle(t.triangle);
        }

      private:
        Renderer *renderer;
    };

    boost::shared_ptr<Shader> ambientShader, lightShader;
    unsigned activeTexture;

    const RenderVisitor renderVisitor;
};

}

#endif /* ZOOM_RENDERER_H_ */
