/*
 * ShadowVolume.cpp
 *
 * Copyright (C) 2009 Matthias Schiffer <matthias@gamezock.de>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include "ShadowVolume.h"
#include "MathUtil.h"

namespace Zoom {

ShadowVolume::ShadowVolume(const Triangle &t, const vmml::vec3f &lightPos) {
  MathUtil::Plane trianglePlane(t);

  for(int i = 0; i < 3; ++i) {
    rays[i].p = t.getVertex(i);
  }

  if(trianglePlane.isBehind(lightPos)) {
    for(int i = 0; i < 3; ++i) {
      rays[i].dir = vmml::vec3f::ZERO;
    }

    return;
  }

  for(int i = 0; i < 3; ++i) {
    rays[i].dir = rays[i].p - lightPos;
  }
}

}
